@@ -3,7 +3,7 @@
  *
  * Standard internal include file for Extended Tcl.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1996 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1997 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclExtdInt.h,v 8.0 1996/11/21 00:23:58 markd Exp $
+ * $Id: tclExtdInt.h,v 8.19 1997/08/10 22:18:27 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -33,11 +33,20 @@
  */
 #include "tclInt.h"
 
+/*
+ * Assert macro for use in TclX.  It is controlled by our own #define, not
+ * NDEBUG, but other than that, it is a wrapper around assert.
+ */
+#ifdef TCLX_DEBUG
+#   define TCLX_ASSERT(expr) (assert (expr))
+#else
+#   define TCLX_ASSERT(expr)
+#endif
+
 /*
  * Get ranges of integers and longs.
  * If no MAXLONG, assume sizeof (long) == sizeof (int).
  */
-
 #ifndef MAXINT
 #    ifdef INT_MAX	/* POSIX */
 #        define MAXINT INT_MAX
@@ -91,7 +100,7 @@ typedef struct {
 } Tcl_SubMatchInfo [NSUBEXP];
 
 /*
- * Flags used by TclX_RegExpCompile:
+ * Flags used by TclX_RegExpCompileObj:
  */
 #define TCLX_REXP_NO_CASE         1   /* Do matching regardless of case    */
 #define TCLX_REXP_BOTH_ALGORITHMS 2   /* Use boyer-moore along with regexp */
@@ -158,8 +167,10 @@ typedef struct {
 
 /*
  * Used to return argument messages by most commands.
+ * FIX: Should be internal, got thought TclX_WrongArgs.
  */
 extern char *tclXWrongArgs;
+extern Tcl_Obj *tclXWrongArgsObj;
 
 /*
  * Macros to do string compares.  They pre-check the first character before
@@ -171,6 +182,11 @@ extern char *tclXWrongArgs;
 #define STRNEQU(str1, str2, cnt) \
         (((str1) [0] == (str2) [0]) && (strncmp (str1, str2, cnt) == 0))
 
+#define OBJSTREQU(obj1, str1) \
+	(strcmp (Tcl_GetStringFromObj (obj1, NULL), str1) == 0)
+
+#define OBJSTRNEQU(obj1, str1, cnt) \
+	(strncmp (Tcl_GetStringFromObj (obj1, NULL), str1, cnt) == 0)
 /*
  * Macro to do ctype functions with 8 bit character sets.
  */
@@ -179,11 +195,15 @@ extern char *tclXWrongArgs;
 #define ISLOWER(c) (islower ((unsigned char) c))
 
 /*
- * Macro that behaves like strdup, only uses ckalloc.
+ * Macro that behaves like strdup, only uses ckalloc.  Also macro that does the
+ * same with a string that might contain zero bytes,
  */
 #define ckstrdup(sourceStr) \
   (strcpy (ckalloc (strlen (sourceStr) + 1), sourceStr))
 
+#define ckbinstrdup(sourceStr, length) \
+  ((char *) memcpy (ckalloc (length + 1), sourceStr, length + 1))
+
 /*
  * Callback type for walking directories.
  */
@@ -197,10 +217,28 @@ typedef int
 /*
  * Prototypes for utility procedures.
  */
+Tcl_Obj *
+TclX_ObjGetVar2S _ANSI_ARGS_((Tcl_Interp *interp,
+                              char *part1Ptr,
+                              char *part2Ptr,
+                              int flags));
+
+extern Tcl_Obj *
+TclX_ObjSetVar2S _ANSI_ARGS_((Tcl_Interp *interp,
+                              char *part1Ptr,
+                              char *part2Ptr,
+                              Tcl_Obj *newValuePtr,
+                              int flags));
+
 extern int
-Tcl_StrToOffset _ANSI_ARGS_((CONST char *string,
-                             int         base,
-                             off_t      *offsetPtr));
+TclX_StrToOffset _ANSI_ARGS_((CONST char *string,
+                              int         base,
+                              off_t      *offsetPtr));
+
+int
+TclX_GetUnsignedFromObj _ANSI_ARGS_((Tcl_Interp *interp,
+                                     Tcl_Obj    *objPtr,
+                                     unsigned   *valuePtr));
 
 extern int
 TclX_Eval _ANSI_ARGS_((Tcl_Interp  *interp,
@@ -218,27 +256,39 @@ TclX_WriteStr _ANSI_ARGS_((Tcl_Channel  channel,
 
 
 extern int
-TclX_GetChannelOption _ANSI_ARGS_((Tcl_Channel channel,
-                                   int         option));
+TclX_GetChannelOption _ANSI_ARGS_((Tcl_Interp  *interp,
+                                   Tcl_Channel  channel,
+                                   int          option,
+                                   int         *valuePtr));
+
+extern Tcl_Obj *
+TclXGetHostInfo _ANSI_ARGS_((Tcl_Interp *interp,
+                             Tcl_Channel channel,
+                             int         remoteHost));
 
 extern Tcl_Channel
 TclX_GetOpenChannel _ANSI_ARGS_((Tcl_Interp *interp,
                                  char       *handle,
-                                 int         direction));
+                                 int         chanAccess));
+
+extern Tcl_Channel
+TclX_GetOpenChannelObj _ANSI_ARGS_((Tcl_Interp *interp,
+                                    Tcl_Obj    *handle,
+                                    int         chanAccess));
 
 extern int
-Tcl_GetOffset _ANSI_ARGS_((Tcl_Interp *interp,
-                           CONST char *string,
-                           off_t      *offsetPtr));
+TclX_GetOffsetFromObj _ANSI_ARGS_((Tcl_Interp *interp,
+                                   Tcl_Obj    *objPtr,
+                                   off_t      *offsetPtr));
 
 extern void
 TclX_RegExpClean _ANSI_ARGS_((TclX_regexp *regExpPtr));
 
 extern int
-TclX_RegExpCompile _ANSI_ARGS_((Tcl_Interp   *interp,
-                                TclX_regexp  *regExpPtr,
-                                char         *expression,
-                                int          flags));
+TclX_RegExpCompileObj _ANSI_ARGS_((Tcl_Interp   *interp,
+                                   TclX_regexp  *regExpPtr,
+                                   Tcl_Obj      *expression,
+                                   int          flags));
 
 extern int
 TclX_RegExpExecute _ANSI_ARGS_((Tcl_Interp       *interp,
@@ -249,10 +299,10 @@ TclX_RegExpExecute _ANSI_ARGS_((Tcl_Interp       *interp,
 
 
 extern int
-Tcl_RelativeExpr _ANSI_ARGS_((Tcl_Interp  *interp,
-                              char        *cstringExpr,
-                              long         stringLen,
-                              long        *exprResultPtr));
+TclX_RelativeExpr _ANSI_ARGS_((Tcl_Interp  *interp,
+                               Tcl_Obj     *exprPtr,
+                               int          stringLen,
+                               int         *exprResultPtr));
 
 extern int
 TclXRuntimeInit _ANSI_ARGS_((Tcl_Interp *interp,
@@ -271,293 +321,111 @@ TclX_JoinPath _ANSI_ARGS_((char        *path1,
                            char        *path2,
                            Tcl_DString *joinedPath));
 
-/*
- * Definitions required to initialize all extended commands.  These are either
- * the command executors or initialization routines that do the command
- * initialization.  The initialization routines are used when there is more
- * to initializing the command that just binding the command name to the
- * executor.  Usually, this means initializing some command local data via
- * the ClientData mechanism.  The command executors should be declared to be of
- * type `Tcl_CmdProc', but this blows up some compilers, so they are declared
- * with an ANSI prototype.
- */
+extern int  
+TclX_WrongArgs _ANSI_ARGS_((Tcl_Interp *interp, 
+                            Tcl_Obj    *commandNameObj, 
+			    char       *string));
 
-/*
- * from tclXbsearch.c
- */
-extern int 
-Tcl_BsearchCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-/*
- * from tclXchmod.c
- */
-extern int 
-Tcl_ChmodCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_ChownCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_ChgrpCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-/*
- * from tclXcmdloop.c
- */
-extern int 
-Tcl_CommandloopCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-/*
- * from tclXdebug.c
- */
 extern void
-Tcl_InitDebug _ANSI_ARGS_((Tcl_Interp *interp));
-
-/*
- * from tclXdup.c
- */
-extern int 
-Tcl_DupCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-/*
- * from tclXfcntl.c
- */
-extern int 
-Tcl_FcntlCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-/*
- * from tclXfilecmds.c
- */
-extern int 
-Tcl_PipeCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_CopyfileCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_LgetsCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+TclX_AppendObjResult _ANSI_ARGS_(TCL_VARARGS_DEF (Tcl_Interp *,arg1));
 
 extern int
-Tcl_FtruncateCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+TclX_IsNullObj _ANSI_ARGS_((Tcl_Obj *objPtr));
+
+void
+TclX_ShellExit _ANSI_ARGS_((Tcl_Interp *interp,
+                            int         exitCode));
 
-extern int
-Tcl_ReaddirCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
 /*
- * from tclXfstat.c
+ * Definitions required to initialize all extended commands.
  */
-extern int 
-Tcl_FstatCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
 
-/*
- * from tclXflock.c
- */
-extern int
-Tcl_FlockCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int
-Tcl_FunlockCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-/*
- * from tclXfilescan.c
- */
 extern void
-Tcl_InitFilescan _ANSI_ARGS_((Tcl_Interp *interp));
+TclX_BsearchInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-/*
- * from tclXgeneral.c
- */
+extern void
+TclX_ChmodInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-extern int 
-Tcl_EchoCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+extern void
+TclX_CmdloopInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-extern int 
-Tcl_InfoxCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+extern void
+TclX_DebugInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-extern int 
-Tcl_LoopCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+extern void
+TclX_DupInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-/*
- * from tclXid.c
- */
-extern int 
-Tcl_IdCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+extern void
+TclX_FcntlInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-/*
- * from tclXkeylist.c
- */
-extern int 
-Tcl_KeyldelCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+extern void
+TclX_FilecmdsInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-extern int 
-Tcl_KeylgetCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+extern void
+TclX_FstatInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-extern int 
-Tcl_KeylkeysCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+extern void
+TclX_FlockInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-extern int 
-Tcl_KeylsetCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
+extern void
+TclX_FilescanInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-/*
- * from tclXlib.c
- */
-extern int 
+extern void
+TclX_GeneralInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+extern void
+TclX_IdInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+extern void
+TclX_KeyedListInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+extern void
+TclX_LgetsInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+extern void
+TclX_ListInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+extern void
+TclX_MathInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+extern void
+TclX_MsgCatInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+extern void
+TclX_ProcessInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+void
+TclX_ProfileInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+extern void
+TclX_RestoreResultErrorInfo _ANSI_ARGS_((Tcl_Interp *interp,
+                                         Tcl_Obj    *saveObjPtr));
+
+extern Tcl_Obj *
+TclX_SaveResultErrorInfo _ANSI_ARGS_((Tcl_Interp  *interp));
+
+extern void
+TclX_SelectInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+extern void
+TclX_SignalInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+extern void
+TclX_StringInit _ANSI_ARGS_((Tcl_Interp *interp));
+
+extern void
 TclX_LibraryInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-/*
- * from tclXlist.c
- */
-extern int 
-Tcl_LvarpopCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_LvarcatCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_LvarpushCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_LemptyCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_LassignCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_LmatchCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_LcontainCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-/*
- * from tclXmath.c
- */
 extern void
-Tcl_InitMath _ANSI_ARGS_((Tcl_Interp*));
+TclX_SocketInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-/*
- * from tclXmsgcat.c
- */
 extern void
-Tcl_InitMsgCat _ANSI_ARGS_((Tcl_Interp *interp));
+TclX_OsCmdsInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-/*
- * from tclXprocess.c
- */
-extern int 
-Tcl_ExeclCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_ForkCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_WaitCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-/*
- * from tclXprofile.c
- */
-void
-Tcl_InitProfile _ANSI_ARGS_((Tcl_Interp *interp));
-
-/*
- * from tclXselect.c
- */
-extern int 
-Tcl_SelectCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-/*
- * from tclXsignal.c
- */
 extern void
-Tcl_InitSignalHandling _ANSI_ARGS_((Tcl_Interp *interp));
+TclX_PlatformCmdsInit _ANSI_ARGS_((Tcl_Interp *interp));
 
-/*
- * from tclXstring.c
- */
-extern int 
-Tcl_CindexCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_ClengthCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int
-Tcl_CconcatCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_CrangeCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_CcollateCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_ReplicateCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_TranslitCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_CtypeCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_CtokenCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_CequalCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-/*
- * from tclXlib.c
- */
-extern void
-Tcl_InitLibrary _ANSI_ARGS_((Tcl_Interp *interp));
-
-/*
- * from tclXoscmds.c
- */
-extern int 
-Tcl_AlarmCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_LinkCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_NiceCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_SleepCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_SyncCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_SystemCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_UmaskCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-/*
- * from tclXunixCmds.c or tclXwinCmds.c
- */
-extern int 
-Tcl_ChrootCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-extern int 
-Tcl_TimesCmd _ANSI_ARGS_((ClientData, Tcl_Interp*, int, char**));
-
-
-/*
- * from tclXsocket.c
- */
-extern int
-TclXGetHostInfo _ANSI_ARGS_((Tcl_Interp *interp,
-                             Tcl_Channel channel,
-                             int         remoteHost));
-
-extern int
-Tcl_HostInfoCmd _ANSI_ARGS_((ClientData  clientData,
-                             Tcl_Interp *interp,
-                             int         argc,
-                             char      **argv));
-
-/*
- * from tclXunixSock.c and stubbed in tclXwinCmds.c.
- */
 extern void
 TclX_ServerInit _ANSI_ARGS_((Tcl_Interp *interp));
 
@@ -572,7 +440,7 @@ TclXOSDupChannel _ANSI_ARGS_((Tcl_Interp *interp,
 
 Tcl_Channel
 TclXOSBindOpenFile _ANSI_ARGS_((Tcl_Interp *interp,
-                                char       *fileNumStr));
+                                int         fileNum));
 
 /*
  * from tclXxxxOS.c
@@ -580,6 +448,9 @@ TclXOSBindOpenFile _ANSI_ARGS_((Tcl_Interp *interp,
 extern int
 TclXNotAvailableError _ANSI_ARGS_((Tcl_Interp *interp,
                                    char       *funcName));
+extern int
+TclXNotAvailableObjError _ANSI_ARGS_((Tcl_Interp *interp,
+				      Tcl_Obj *obj));
 
 extern clock_t
 TclXOSTicksToMS _ANSI_ARGS_((clock_t numTicks));
@@ -643,7 +514,6 @@ TclXOSkill _ANSI_ARGS_((Tcl_Interp *interp,
 extern int
 TclXOSFstat _ANSI_ARGS_((Tcl_Interp  *interp,
                          Tcl_Channel  channel,
-                         int          direction,
                          struct stat *statBuf,
                          int         *ttyDev));
     
@@ -661,7 +531,6 @@ TclXOSWalkDir _ANSI_ARGS_((Tcl_Interp       *interp,
 
 extern int
 TclXOSGetFileSize _ANSI_ARGS_((Tcl_Channel  channel,
-                               int          direction,
                                off_t       *fileSize));
 
 extern int
@@ -672,7 +541,7 @@ TclXOSftruncate _ANSI_ARGS_((Tcl_Interp  *interp,
 
 extern int
 TclXOSfork _ANSI_ARGS_((Tcl_Interp *interp,
-                        char       *funcName));
+                        Tcl_Obj    *funcNameObj));
 
 extern int
 TclXOSexecl _ANSI_ARGS_((Tcl_Interp *interp,
@@ -718,28 +587,27 @@ TclXOSfchmod _ANSI_ARGS_((Tcl_Interp *interp,
                           Tcl_Channel channel,
                           int         mode,
                           char       *funcName));
+extern int  
+TclXOSChangeOwnGrpObj _ANSI_ARGS_((Tcl_Interp  *interp,
+                                   unsigned     options,
+                                   char        *ownerStr,
+                                   char        *groupStr,
+                                   Tcl_Obj     *fileList,
+                                   char        *funcName));
 
 extern int
-TclXOSChangeOwnGrp _ANSI_ARGS_((Tcl_Interp  *interp,
-                                unsigned     options,
-                                char        *ownerStr,
-                                char        *groupStr,
-                                char       **files,
-                                char       *funcName));
-
-extern int
-TclXOSFChangeOwnGrp _ANSI_ARGS_((Tcl_Interp *interp,
-                                 unsigned    options,
-                                 char       *ownerStr,
-                                 char       *groupStr,
-                                 char      **channelIds,
-                                 char       *funcName));
+TclXOSFChangeOwnGrpObj _ANSI_ARGS_((Tcl_Interp *interp,
+                                    unsigned    options,
+                                    char       *ownerStr,
+                                    char       *groupStr,
+                                    Tcl_Obj    *channelIdList,
+                                    char       *funcName));
 
 int
 TclXOSGetSelectFnum _ANSI_ARGS_((Tcl_Interp *interp,
                                  Tcl_Channel channel,
-                                 int        *readFnumPtr,
-                                 int        *writeFnumPtr));
+                                 int         direction,
+                                 int        *fnumPtr));
 
 int
 TclXOSHaveFlock _ANSI_ARGS_((void));
@@ -772,3 +640,5 @@ TclXOSSetCloseOnExec _ANSI_ARGS_((Tcl_Interp *interp,
                                   Tcl_Channel channel,
                                   int         value));
 #endif
+
+
